
/********************************************
 **** Animation Class Library V1.0  1994 Yves Schmid & Alia Development
 ****
 **** AnimCObject.h
 ****
 **** Created:      28 May 1994
 **** Modified:     01 Septembre 1994
 **** Version:      0
 **** Compatible:   C++, Mac System 7
 ****
 **** Description:	Animation objects which handle controls.
 ****
 ****				AnimCObject is a child class of the "AnimObject" class.
 ****
 *******************/
 
#ifndef AnimCObject_H
#define AnimCObject_H


#include "CoreHead.h"
#include "AnimControl.h"
#include "AnimObject.h"


class AnimBase;

class AnimCObject: public AnimObject
{

	//***********************************************************
	//.............. P U B L I C   M E T H O D S.................

	public:

	AnimCObject(AnimSupervisor *base);
	~AnimCObject(void);

	
	
	virtual void runcontrol(AnimControl *control);	// Installs a new control, the previous
												   // control is removed
												
	virtual void stopcontrol(void);		// Removes immediatly the current control
	
	inline AnimControl *getcurrentcontrol(void) {return control;}	// Returns the current
																   // control or NULL									



	//***********************************************************

	protected:

	virtual void processcontrols(void);
	

	virtual void initonecontrol(AnimControl *);			// Called at the start of
														// a new control
																	
	virtual Boolean processonecontrol(AnimControl *);	// Process a control, override this
														// method to process new
														// commands


	private:
	
	AnimControl	*control;				
	Boolean		controlrunning;			
	float		controlspeed;			
	long		controlwait;			

	float		sign_x,sign_y;			
	float		slope,x,y;
	float		x1,x2,y1,y2,f;
	Boolean		checkangle45;

	float		curveloop;				
	float		Qx,Qy,Px,Py,Rx,Ry,cx,cy;
	float		curvespeed;
	Boolean		dostartgoto;

	void	startgoto(float ax1, float ay1, float ax2, float ay2);
	Boolean processgoto(float sx, float sy);

	void	startcurve(float ax1, float ay1, float ax2, float ay2, float mx, float my);
	Boolean processcurve(float sx, float sy);
	Boolean	processcurvegoto(float sx, float sy);
	
};



#endif

